package de.c64_editor.model;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;

/**
 * Stellt Methoden zum Schreiben und Lesen von Byte-Arrays in und aus Dateien
 * zur Verfuegung.
 *
 * @author Argh The Awful
 */
public class FileIO {

	/**
	 * Schreibt ein Array vom Typ byte in eine Datei.
	 *
	 * @param fileName Name der zu schreibenden Datei.
	 * @param buffer zu schreibendes Array.
	 */
	@SuppressWarnings("unused")
	public static void writeByteArray(String fileName, byte[] buffer)
															throws IOException{
		writeByteArray(new File(fileName), buffer);
	}

	/**
	 * Schreibt ein Array vom Typ byte in eine Datei.
	 *
	 * @param file Die Datei, in die geschrieben werden soll
	 * @param buffer zu schreibendes Array
	 */
	public static void writeByteArray(File file, byte[] buffer)
															throws IOException {
		FileOutputStream outputStream = new FileOutputStream(file);
		outputStream.write(buffer);
		outputStream.close();
	}

	/**
	 * Liest Bytes aus einer Datei und packt diese in einen Array vom Typ byte.
	 *
	 * @param fileName Name der Datei, aus der gelesen werden soll.
	 * @param buffer Array, in das die Daten aus der Datei geschrieben werden
	 *               sollen.
	 */
	@SuppressWarnings("unused")
	public static void readByteArray(String fileName, byte[] buffer)
															throws IOException {
		readByteArray(new File(fileName), buffer);
	}

	/**
	 * Liest Bytes aus einer Datei und packt diese in einen Array vom Typ byte.
	 *
	 * @param file Die zu landende Datei
	 * @param buffer Array, in das die Daten aus der Datei geschrieben werden
	 *               sollen.
	 */
	public static void readByteArray(File file, byte[] buffer)
															throws IOException {
		FileInputStream inputStream = new FileInputStream(file);
		int length = (int)file.length();
		inputStream.read(buffer, 0, length);
		inputStream.close();
	}
}
