package de.c64_editor.view;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.lang.reflect.Constructor;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.border.EtchedBorder;
import de.c64_editor.charset.C64Colors;

/**
 * The {@link ColorView} implements the view of the 3 base colors for multicolor
 * 
 * @author Lenny
 */
public class ColorView extends JFrame {
	private static final long serialVersionUID = 1L;

	// some int parameters
	private int colors = 16, iconSize = 32, colorCounter, markedColor = 0;

	// The buttons
	private JButton[] colorButtons = new JButton[colors];

	// Create panels
	private JPanel colorPanel = new JPanel(new GridLayout(2, colors / 2));
	private JPanel checkboxPanel = new JPanel(new GridLayout(0, colors / 2));

	private JCheckBox checkbox;

	/**
	 * The default {@link Constructor}.
	 */
	public ColorView() {
		viewInit();
		callbackInit();
		pack();
		setTitle("Farbpalette");
	}

	/**
	 * This initializes all the {@link ActionListener} and acts on
	 * {@link ActionEvent}.
	 */
	private void callbackInit() {
		for (colorCounter = 0; colorCounter < colorButtons.length; colorCounter++) {
			colorButtons[colorCounter].addMouseListener(new MouseListener() {
				int count = colorCounter;

				/*
				 * If a color is clicked, mark the color and put the number of
				 * it in markedColor.
				 */
				public void mouseClicked(MouseEvent e) {
					colorButtons[markedColor].setBorderPainted(false);
					markedColor = count;
					colorButtons[count].setBorderPainted(true);
					colorButtons[count].setBorder(new EtchedBorder(Color.black,
							Color.yellow));
				}

				@Override
				public void mouseEntered(MouseEvent e) {
					// Do nothing
				}

				@Override
				public void mouseExited(MouseEvent e) {
					// Do nothing
				}

				@Override
				public void mousePressed(MouseEvent e) {
					// Do nothing
				}

				@Override
				public void mouseReleased(MouseEvent e) {
					// Do nothing
				}

			});
		}

	}

	/**
	 * This initializes the view.
	 */
	private void viewInit() {
		this.setAlwaysOnTop(true);
		// JTextField checkboxLabel= new JTextField("Nur Farbe ändern");
		this.setLayout(new GridLayout(2, 0));
		this.setDefaultCloseOperation(HIDE_ON_CLOSE);
		checkbox = new JCheckBox("", false);
		this.add(colorPanel);
		this.add(checkboxPanel);
		checkboxPanel.setBackground(Color.black);
		checkboxPanel.add(checkbox);
		// checkboxLabel.setPreferredSize(new Dimension(iconSize*3,iconSize));
		// checkboxPanel.add(checkboxLabel);

		for (int i = 0; i < colorButtons.length; i++) {
			colorButtons[i] = new JButton();
			colorButtons[i].setPreferredSize(new Dimension(iconSize, iconSize));
			colorButtons[i].setBackground(C64Colors.getColor(i));
			colorPanel.add(colorButtons[i]);
		}
		colorButtons[1].setBorder(new EtchedBorder(Color.blue, Color.yellow));
		/*
		 * ItemListener heldenListener = new ItemListener() { public void
		 * itemStateChanged( ItemEvent e ) { } };
		 */

		/*
		 * checkbox.addItemListener( heldenListener );
		 */

	}

	/**
	 * Getter for the choosed color.
	 * 
	 * @return the markedColor
	 */
	public int getMarkedColor() {
		return markedColor;
	}

	/**
	 * @return the checkbox
	 */
	public boolean getCheckbox() {
		return checkbox.isSelected();
	}

	/**
	 * Setter for markedColor. If the markedColor will be set, the view will be
	 * mark the new color too.
	 * 
	 * @param markedColor
	 *            the markedColor to set
	 */
	public void setMarkedColor(int newMarkedColor) {
		colorButtons[markedColor].setBorderPainted(false);
		markedColor = newMarkedColor;
		colorButtons[markedColor].setBorderPainted(true);
		colorButtons[markedColor].setBorder(new EtchedBorder(Color.black,
				Color.yellow));
	}

}
