package de.c64_editor.view;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.io.File;
import java.io.IOException;
import java.lang.reflect.Constructor;
import java.util.Arrays;

import javax.swing.BorderFactory;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.JViewport;
import javax.swing.border.EtchedBorder;
import javax.swing.filechooser.FileFilter;

import net.java.balloontip.BalloonTip;
import net.java.balloontip.styles.RoundedBalloonStyle;
import net.java.balloontip.utils.TimingUtils;

import de.c64_editor.charset.C64CharSet;
import de.c64_editor.model.FileIO;

/**
 * The {@link MainView} implements the levelPanel and the charPanel.
 * 
 * @author Lenny
 */
public class MainView extends JFrame {
	private static final long serialVersionUID = 1L;

	private boolean clickableLabels = false;
	private ColorView colorView;

	// The charset
	private C64CharSet charSet;

	// some int parameters
	private int levelWidth = 512, levelHeight = 16, charsetWidth = 32,
			charsetHeight = 8, levelByteLength = 8192, iconSize = 32, d021 = 0,
			d022 = 12, d023 = 11, d021Default = 0, d022Default = 12,
			d023Default = 11, bla = 1;

	// which char is clicked
	private int charCounter;
	private int markedChar;

	// which level label is clicked
	private int levelCounter;

	// The resolution of the monitor
	private int width = new Double(Toolkit.getDefaultToolkit().getScreenSize()
			.getWidth()).intValue() / 2, height = new Double(Toolkit
			.getDefaultToolkit().getScreenSize().getHeight()).intValue() / 2;

	// Create the menu bar
	private JMenuBar menuBar = new JMenuBar();
	private JMenu menuDatei = new JMenu("Datei");
	private JMenuItem itemGridsize = new JMenuItem("Gridgröße");
	private JMenuItem itemQuit = new JMenuItem("Beenden");
	private JMenuItem itemBuild = new JMenuItem("Level erstellen");
	private JMenuItem itemLoadCharset = new JMenuItem("Charset laden");
	private JMenuItem itemLoadLevel = new JMenuItem("Level laden");
	private JMenuItem itemColor = new JMenuItem("Farbpalette");
	private JMenuItem itemBaseColor = new JMenuItem("Grundfarben einstellen");

	// Create panels
	private JPanel levelPanel = new JPanel(new GridLayout(levelHeight,
			levelWidth));
	private JPanel charPanel = new JPanel(new GridLayout(charsetHeight,
			charsetWidth));
	private JScrollPane levelScrollpane = new JScrollPane(
			JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
			JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
	private JScrollPane charScrollpane = new JScrollPane();
	private JViewport levelViewport = new JViewport();
	private JViewport charViewport = new JViewport();

	// Everything to save the level file
	private File levelFile;
	// private FileOutputStream fileOut;

	// Everything to load the charset file
	private File charsetFile;
	private File levelLoadFile;
	private byte[] buildedLevelChars;
	private byte[] buildedLevelColors;

	// create the levellabels[], charlabels[] and initialize
	private JLabel[] levelLabels = new JLabel[levelByteLength];
	private JLabel[] columnLabels = new JLabel[levelWidth];
	private JLabel[] charLabels = new JLabel[256];

	// create the balloon tips
	private BalloonTip[] columnTip = new BalloonTip[levelByteLength];
	private RoundedBalloonStyle roundedBalloonStyle = new RoundedBalloonStyle(
			10, 10, Color.lightGray, Color.blue);

	// the level
	byte[] levelChars = new byte[levelByteLength];
	byte[] levelColors = new byte[levelByteLength];
	byte[] levelFull = new byte[levelByteLength * 2 + 3];

	// the charSet
	byte[] charSetBuffer = new byte[2304];

	/**
	 * This is the default {@link Constructor}.
	 */
	public MainView() {
		// initialize the level[], levelColors[], levelFull[] with 0
		Arrays.fill(levelChars, (byte) 0);
		Arrays.fill(levelColors, (byte) 0);
		Arrays.fill(levelFull, (byte) 0);

		viewInit();
		callbackInit();
		setSize(width, height);
		// setExtendedState(Frame.MAXIMIZED_BOTH);
		setTitle("ByteLevel 0.3 Alpha - C64 Level Editor");
	}

	/**
	 * This initializes all the {@link ActionListener} and acts on
	 * {@link ActionEvent}.
	 */
	private void callbackInit() {

		// if Quit is clicked, close the app
		itemQuit.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				System.exit(EXIT_ON_CLOSE);
			}
		});

		// if gridsize is clicked, open the GridChooseView.
		itemGridsize.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				String s = JOptionPane
						.showInputDialog("Welche Länge soll das Level haben?");
				levelWidth = Integer.valueOf(s).intValue();
				// TODO View mit neuer Gridsize updaten
			}
		});

		// if baseColor is choosed, open the option dialog
		itemBaseColor.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				JTextField d021TextField = new JTextField(String.valueOf(d021));
				JTextField d022TextField = new JTextField(String.valueOf(d022));
				JTextField d023TextField = new JTextField(String.valueOf(d023));
				Object[] message = { "$D021", d021TextField, "$D022",
						d022TextField, "$D023", d023TextField };

				Object[] options = { "OK", "Default", "Abbrechen" };

				JOptionPane pane = new JOptionPane(message,
						JOptionPane.PLAIN_MESSAGE);
				pane.setOptions(options);
				pane.createDialog(null, "3 Grundfarben").setVisible(true);

				if (pane.getValue() == options[0]) {
					try {
						if ((Integer.parseInt(d021TextField.getText()) >= 0)
								&& (Integer.parseInt(d022TextField.getText()) >= 0)
								&& (Integer.parseInt(d023TextField.getText()) >= 0)
								&& (Integer.parseInt(d021TextField.getText()) <= 15)
								&& (Integer.parseInt(d022TextField.getText()) <= 15)
								&& (Integer.parseInt(d023TextField.getText()) <= 15)) {
							d021 = Integer.valueOf(d021TextField.getText())
									.intValue();
							d022 = Integer.parseInt(d022TextField.getText());
							d023 = Integer.parseInt(d023TextField.getText());
							setCharIcons();
							setLevelIcons();
						} else {
							JOptionPane.showMessageDialog(null,
									"Jede Farbe muss einen Wert zwischen 0"
											+ " und 15 haben!", "Achtung",
									JOptionPane.WARNING_MESSAGE);
						}
					} catch (Exception e2) {
						JOptionPane.showMessageDialog(null,
								"Es dürfen nur Integer eingegeben werden!",
								"Achtung", JOptionPane.WARNING_MESSAGE);
					}
				}
				if (pane.getValue() == options[1]) {
					System.out.println("Default");
					d021 = d021Default;
					d022 = d022Default;
					d023 = d023Default;
					setCharIcons();
					setLevelIcons();
				}
			}
		});

		// if colorItem is clicked, show the ColorView.
		itemColor.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				colorView.setVisible(true);
			}
		});

		// if LoadLevel is clicked, open a load dialog
		itemLoadLevel.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent e) {

				// load the charset from a file
				JFileChooser fileChooser = new JFileChooser();
				fileChooser.setFileFilter(new FileFilter() {

					@Override
					public boolean accept(File f) {
						return true;
						// return f.getName().toLowerCase().endsWith(".lvl");
					}

					@Override
					public String getDescription() {
						return "C64-Level";
					}
				});

				fileChooser.setCurrentDirectory(new File("."));
				int state = fileChooser.showOpenDialog(null);

				if (state == JFileChooser.APPROVE_OPTION) {
					levelLoadFile = fileChooser.getSelectedFile();
					byte[] buffer = new byte[levelByteLength * 2 + 3];
					try {
						FileIO.readByteArray(levelLoadFile, buffer);
					} catch (IOException ex) {
						System.out.println(ex.getMessage());
					}
					loadBuildedLevel(buffer);
					setLevelIcons();
				}
			}
		});

		// if LoadCharset is clicked, open a load dialog
		itemLoadCharset.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent e) {

				// load the charset from a file
				JFileChooser fileChooser = new JFileChooser();
				fileChooser.setFileFilter(new FileFilter() {

					@Override
					public boolean accept(File f) {
						return true;
						// return f.getName().toLowerCase().endsWith(".prg");
					}

					@Override
					public String getDescription() {
						return "C64-Charset";
					}
				});
				fileChooser.setCurrentDirectory(new File("."));
				int state = fileChooser.showOpenDialog(null);

				if (state == JFileChooser.APPROVE_OPTION) {
					charsetFile = fileChooser.getSelectedFile();
					byte[] buffer = new byte[2304];
					try {
						FileIO.readByteArray(charsetFile, buffer);
					} catch (IOException ex) {
						System.out.println(ex.getMessage());
					}
					charSet = new C64CharSet(buffer);
					charSetBuffer = buffer;
					setCharIcons();
					itemBuild.setEnabled(true);
					clickableLabels = true;
					itemLoadLevel.setEnabled(true);
				}
			}
		});

		// if Build is clicked, open a save as dialog
		itemBuild.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				JFileChooser fileChooser = new JFileChooser();
				fileChooser.setFileFilter(new FileFilter() {

					@Override
					public boolean accept(File level) {
						return true;
						// return levelFile.isDirectory()
						// || levelFile.getName().toLowerCase().endsWith(
						// ".lvl");
					}

					@Override
					public String getDescription() {
						return "C64 Level";
					}

				});
				fileChooser.setCurrentDirectory(new File("."));
				int result = fileChooser.showSaveDialog(null);

				if (result == JFileChooser.APPROVE_OPTION) {
					levelFile = fileChooser.getSelectedFile();
				}

				buildLevel();
				try {
					FileIO.writeByteArray(levelFile, levelFull);
				} catch (IOException ex) {
					System.out.println(ex.getMessage());
				}
			}
		});

		// if a level label is clicked with the left button, place the char
		// icon in the level label or if it is clicked with the right
		// button, clear the icon
		for (levelCounter = 0; levelCounter < levelLabels.length; levelCounter++) {
			levelLabels[levelCounter].addMouseListener(new MouseListener() {
				int count = levelCounter;

				@Override
				public void mouseClicked(MouseEvent e) {
					if (e.getButton() == MouseEvent.BUTTON1 && clickableLabels
							&& colorView.getCheckbox()) {
						System.out.println("an");
						System.out.println(levelChars[count]);

						// switch the icon
						levelLabels[count].setIcon(charSet.getChar(
								levelChars[count]).getMCIcon(4, d021, d022,
								d023, colorView.getMarkedColor()));

						// put the number of the char in the levelChars[]
						// levelChars[count] = (byte) markedChar;

						// put the number of the marked color in the
						// levelColors[]
						levelColors[count] = (byte) colorView.getMarkedColor();

						System.out.println("du hast label " + (count + 1)
								+ " geklickt\n Der Inhalt ist jetzt"
								+ levelChars[count]);
						System.out.println("farbe="+levelColors[count]);
					}
					if (e.getButton() == MouseEvent.BUTTON1 && clickableLabels
							&& !colorView.getCheckbox()) {
						System.out.println("aus");
						// switch the icon
						levelLabels[count].setIcon(charSet.getChar(markedChar)
								.getMCIcon(4, d021, d022, d023,
										colorView.getMarkedColor()));

						// put the number of the char in the levelChars[]
						levelChars[count] = (byte) markedChar;

						// put the number of the marked color in the
						// levelColors[]
						levelColors[count] = (byte) colorView.getMarkedColor();

						System.out.println("du hast label " + (count + 1)
								+ " geklickt\n Der Inhalt ist jetzt"
								+ levelChars[count]);
						System.out.println("farbe="+levelColors[count]);
					}
					if (e.getButton() == MouseEvent.BUTTON3 && clickableLabels) {

						// delete the icon and the bytecode in the level
						levelLabels[count].setIcon(null);

						levelChars[count] = 0;

						System.out.println("label " + count
								+ " hat nun den inhalt " + levelChars[count]
								+ bla);
					}
					if (e.getButton() == MouseEvent.BUTTON2 && clickableLabels) {
						if (clickableLabels) {
							columnTip[count] = new BalloonTip(
									levelLabels[count], String
											.valueOf((count + 1) % levelWidth),
									roundedBalloonStyle, false);
							TimingUtils.showTimedBalloon(columnTip[count], 750);
							// System.out.println("tip " + count);
						}
					}

					bla++;
				}

				@Override
				public void mouseEntered(MouseEvent e) {
					if (count < levelWidth && clickableLabels) {
						columnTip[count] = new BalloonTip(levelLabels[count],
								String.valueOf(count + 1), roundedBalloonStyle,
								false);
						TimingUtils.showTimedBalloon(columnTip[count], 750);
						// System.out.println("tip " + count);
					}
				}

				@Override
				public void mouseExited(MouseEvent e) {
					if (count < levelWidth && clickableLabels) {
						columnTip[count].closeBalloon();
					}
				}

				@Override
				public void mousePressed(MouseEvent e) {
					// do nothing
				}

				@Override
				public void mouseReleased(MouseEvent e) {
					// do nothing
				}
			});
		}

		// if a char label is clicked, store the index number in
		// choosedchar
		for (charCounter = 0; charCounter < charLabels.length; charCounter++) {
			charLabels[charCounter].addMouseListener(new MouseListener() {
				int count = charCounter;

				@Override
				public void mouseClicked(MouseEvent e) {
					if (clickableLabels) {
						charLabels[markedChar].setBorder(BorderFactory
								.createLineBorder(Color.lightGray));
						System.out.println("du hast char " + count
								+ " angeklickt.");
						markedChar = count;
						charLabels[markedChar].setBorder(new EtchedBorder(
								Color.black, Color.yellow));
						colorView
								.setMarkedColor(charSetBuffer[markedChar + 2048]);
					}
				}

				@Override
				public void mouseEntered(MouseEvent e) {
					// do nothing
				}

				@Override
				public void mouseExited(MouseEvent e) {
					// do nothing
				}

				@Override
				public void mousePressed(MouseEvent e) {
					// do nothing
				}

				@Override
				public void mouseReleased(MouseEvent e) {
					// do nothing
				}
			});

		}
	}

	/**
	 * This initializes the view.
	 */
	private void viewInit() {
		this.setLayout(new GridLayout(2, 0));
		this.setDefaultCloseOperation(EXIT_ON_CLOSE);

		// Create sizes of the panels
		// levelPanel.setPreferredSize(new Dimension(12800, 512));
		levelPanel.setBackground(Color.BLACK);

		// double buffering JPanels
		levelPanel.isDoubleBuffered();
		charPanel.isDoubleBuffered();

		// add components
		this.add(levelScrollpane);
		levelViewport.add(levelPanel);
		levelScrollpane.setViewport(levelViewport);
		this.add(charScrollpane);
		charViewport.add(charPanel);
		charScrollpane.setViewport(charViewport);

		// Build the menu
		menuDatei.add(itemLoadCharset);
		menuDatei.add(itemLoadLevel);
		menuDatei.add(itemBaseColor);
		menuDatei.add(itemGridsize);
		menuDatei.add(itemColor);
		menuDatei.add(itemBuild);
		menuDatei.add(itemQuit);

		// add the menu to the menu bar
		menuBar.add(menuDatei);

		// add the menuBar to the window
		setJMenuBar(menuBar);

		// create the level labels
		for (int i = 0; i < levelLabels.length; i++) {
			levelLabels[i] = new JLabel("" + (i + 1));
			levelLabels[i].setPreferredSize(new Dimension(iconSize, iconSize));
			levelLabels[i].isOpaque();
			// levelLabels[i].setForeground(Color.lightGray);
			levelLabels[i].setBorder(BorderFactory
					.createLineBorder(Color.lightGray));
		}
		for (int i = 0; i < levelWidth; i++) {
			columnLabels[i] = new JLabel("" + (i + 1));
			columnLabels[i].setPreferredSize(new Dimension(iconSize, iconSize));
			columnLabels[i].isOpaque();
			columnLabels[i].setBorder(BorderFactory
					.createLineBorder(Color.lightGray));
		}

		// put the level labels onto the levelPanel
		for (int i = 0; i < levelLabels.length; i++) {
			levelPanel.add(levelLabels[i]);
		}

		// create the char labels
		for (int i = 0; i < charLabels.length; i++) {
			charLabels[i] = new JLabel("" + (i + 1));
			charLabels[i].setPreferredSize(new Dimension(iconSize, iconSize));
			charLabels[i].setBorder(BorderFactory
					.createLineBorder(Color.lightGray));
		}

		// put the char labels onto the charPanel
		for (int i = 0; i < charLabels.length; i++) {
			charPanel.add(charLabels[i]);
		}

		itemBuild.setEnabled(false);
		itemGridsize.setEnabled(false);
		itemLoadLevel.setEnabled(false);

		this.setVisible(true);
		colorView = new ColorView();
	}

	/**
	 * This puts the char icons onto the char labels.
	 */
	public void setCharIcons() {
		for (int i = 0; i < charLabels.length; i++) {
			charLabels[i].setIcon(charSet.getChar(i).getMCIcon(4, d021, d022,
					d023, charSetBuffer[i + 2048]));
		}
	}

	/**
	 * This puts the level icons onto the level labels.
	 * 
	 * @param buffer
	 */
	public void setLevelIcons() {
		for (int i = 0; i < levelLabels.length; i++) {
			if (levelChars[i] == 0) {
				levelLabels[i].setIcon(null);
			} else {
				levelLabels[i].setIcon(charSet.getChar(levelChars[i])
						.getMCIcon(4, d021, d022, d023, levelColors[i]));
			}
		}
	}

	/**
	 * This builds the buildedLevel[] to save it correctly.
	 */
	public void buildLevel() {
		int add = 0;
		int sum;
		buildedLevelChars = new byte[levelByteLength];
		buildedLevelColors = new byte[levelByteLength];

		// sort the level
		for (int i = 0; i < levelWidth; i++) {
			sum = i;
			buildedLevelChars[add] = levelChars[sum];
			buildedLevelColors[add] = levelColors[sum];
			add++;

			for (int j = 1; j < levelHeight; j++) {
				sum = sum + levelWidth;
				buildedLevelChars[add] = levelChars[sum];
				buildedLevelColors[add] = levelColors[sum];
				add++;
			}
		}

		// fill the level with the chars
		for (int i = 0; i < buildedLevelChars.length; i++) {
			levelFull[i] = buildedLevelChars[i];
		}

		// fill the level with the colors
		for (int i = buildedLevelChars.length; i < levelFull.length - 3; i++) {
			levelFull[i] = buildedLevelColors[i - levelByteLength];
		}

		// put the basecolors of multicolor at the end of the level
		levelFull[levelFull.length - 3] = (byte) d021;
		levelFull[levelFull.length - 2] = (byte) d022;
		levelFull[levelFull.length - 1] = (byte) d023;
	}

	/**
	 * The content of the file will be reordered so that it looks correctly.
	 * 
	 * @param loadedLevel
	 *            the loaded file.
	 */
	public void loadBuildedLevel(byte[] loadedLevel) {
		int sum = 0;
		for (int i = 0; i < levelWidth; i++) {
			levelChars[i] = loadedLevel[sum];
			levelColors[i] = loadedLevel[sum + levelByteLength];
			sum = sum + 1;
			int add = 0;
			for (int j = 1; j < levelHeight; j++) {
				add = j * levelWidth + i;
				levelChars[add] = loadedLevel[sum];
				levelColors[add] = loadedLevel[sum + levelByteLength];
				sum = sum + 1;
			}
		}
		System.out.println(levelChars[10]);
	}
}
