;;==============================================================================
;; Versuch, Scrolling mittels Raster-IRQ zu erzeugen.
;; Hierbei wird der Vector, der normalerweise auf die IRQ-Interrupt-Routine
;; zeigt, auf eine eigene Routine umgebogen.
;; Darueber hinaus wird nur noch der durch den VIC in einer bestimmten
;; Rasterzeile ausgeloeste IRQ zugelassen.
;;==============================================================================
;;==============================================================================
;; Ich frage ausserdem den Joystick in Port 2!!! ab. Bei irgendeiner Aktivitaet
;; veraendere ich die Randfarbe.
;;==============================================================================
!to "demo", cbm

* = $1000

screen_buf0	= $0400		; default-Adresse des Bildschirmspeichers
screen_buf1	= $0800		; dubble-buffer-Adresse des Bildschirmspeichers

cinv		= $0314		; Vector to IRQ Interrupt Routine

scroly		= $D011		; Vertical Fine Scrolling and Control Register
raster		= $D012		; Read Current Raster Scan Line /
				;   Write Line to Compare for Raster IRQ
scrolx		= $D016		; Horizontal Fine Scrolling and Control Register
vmcsb		= $D018		; VIC-II Chip Memory Control Register
vicirq		= $D019		; VIC Interrupt Flag Register
irqmask		= $D01A		; IRQ Mask Register
extcol		= $D020		; Border Color Register
bgcol0		= $D021		; Background Color 0
bgcol1		= $D022		; Background Color 1
bgcol2		= $D023		; Background Color 2
bgcol3		= $D024		; Background Color 3

color_RAM	= $D800		; Color RAM
ciaicr		= $DC0D		; Interrupt Control Register

bsout		= $FFD2		; Zeichen an aktueller Cursor-Pos. ausgeben

tempo		= $03

;;==============================================================================
;; Initialisierung und so.
;;==============================================================================

;	LDA	#$93		; SHIFT + CLR/HOME (Bildschirm loeschen)
;	JSR	bsout		; gibt o.g. Zeichen aus

;	JSR	write_testline	; schreibt was auf Bildschirm zwecks Testen

	LDA	#$00
	STA	extcol		; Randfarbe schwarz
	STA	bgcol0		; Hintergrund schwarz

	LDA	#$07		; 38-Spalten-Modus AN, Multicolor AUS
				; Offset vom li. Bildschirmrand = 7 Pixel
	STA	scrolx

	SEI			; Bevor in CINV geschrieben wird, muessen alle
				; IRQs unterdrueckt werden, damit x
				; Furchtbares passiert.

;;------------------------------------------------------------------------------
;; IRQ-Vector auf eigene Routine umbiegen.
;;------------------------------------------------------------------------------
	LDA	#<init		; LSB der Adresse der init-Routine
	LDX	#>init		; MSB der Adresse der init-Routine
	STA	cinv		; default-IRQ-Zeiger soll auf eigene
	STX	cinv+1		;   Routine zeigen

;;------------------------------------------------------------------------------
;; In den naechsten Zeilen wird festgelegt, in welcher Rasterzeile der Interrupt
;; ausgeloest werden soll.
;;------------------------------------------------------------------------------
	LDA	#$7F		; Maskiert Bits 0-6
	AND	scroly		; Bit 8 des Rasterzeilenvergleichsregisters
	STA	scroly		; loeschen damit nur die Zeilen 0-255 gesetzt
				; werden koennen.
	LDA	#$FF		; Raster-IRQ in Rasterzeile %% ausl�sen
	STA	raster

;;------------------------------------------------------------------------------
;; Saemtliche anderen Interrupts unterdruecken und nur Raster-IRQ zulassen.
;;------------------------------------------------------------------------------
	LDA	#$7F		; Timer A, Timer B, Time of Day Clock,
				; Serial Register und FLAG line
	STA	ciaicr		; werden DEaktivert
	LDA	#$01		; Raster Compare IRQ (und nichts anderes)
	STA	irqmask		; zulassen.

	CLI			; Jetzt koennen die IRQ wieder zugelassen werden.
	JSR	init_sprite0

hold:
	LDA	$DC01
	CMP	#$EF
	BNE	hold
	BRK

;;------------------------------------------------------------------------------
;; Hier ist jetzt die Routine, die beim Rasterinterrupt angesprungen wird (die
;; Adresse, auf $0315/$0315 zeigt).
;;------------------------------------------------------------------------------
init:
	LDA	#$01		; Kapier ich noch nicht, muss aber wohl
	STA	vicirq

	JSR	check_stick1	; Fragt den Joystick in Port 2!!!eins!!elf! ab.
				; Muss mal umbenannt werden.

	LDX	#$01		; Hier kann man die scroll-Geschwindigkeit
				; aendern (1 = 1 Pixel pro Bildaufbau,
				; 2 = Pixel, 3 = 3 Pixel usw).
tempo_loop:
	JSR	scroll
	DEX
	BNE	tempo_loop

	JMP	$EA31		; IRQ Interrupt Entry: Die Standard-IRQs
				; ebenfalls ausfuehren.



;;==============================================================================
;; Rotiert den Bildschirminhalt linksrum (softscroll-maessig).
;;==============================================================================
scroll:
	PHP
	PHA
	TXA
	PHA

	LDA	#$07		; Maskiert die Bits 0-2
	AND	scrolx
	BEQ	switch_page	; Offset vom linken Bildschirmrand = 0 Pixel?

dec_offset:
	DEC	scrolx		; Ansonsten Offset - 1

	LDA	scrolx
	AND	#$07
	CMP	#$04		; Bei einem Offset von 4 Pixeln vom BiSchiRand
				; soll der Inhalt von Puffer X in Puffer Y
				; und um eins nach links verschoben werden.
	BNE	scroll_end	; Ansonten tue nix weiter.

	JSR	save_left
	JSR	scroll_left_2
	JSR	restore_right

	JMP	scroll_end

switch_page:
	LDA	vmcsb
	EOR	#$30		; switcht Bit 4 und 5
	STA	vmcsb

	LDA	scrolx		; Setzt neues Offset fuer Abstand vom linken
	ORA	#$07		; Bildschirmrand
	STA	scrolx


scroll_end:
	LDA	#$80
	STA	$07F8
	STA	$0BF8

	PLA
	TAX
	PLA
	PLP
	RTS



;;==============================================================================
;; Scrollt den ganzen Bildschirm auf einmal (einschliesslich der Sprite-dingsdas
;;==============================================================================
scroll_left_2:
	PHP
	PHA
	TXA
	PHA

	LDX	#$00
scroll_l_2_loop:
	LDA	$401, X			; 4 Cycles
	STA	$800, X			; 5 Cycles
	LDA	$501, X			; 4 Cycles
	STA	$900, X			; 5 Cycles
	LDA	$601, X			; 4 Cycles
	STA	$A00, X			; 5 Cycles
	LDA	$701, X			; 4 Cycles
	STA	$B00, X			; 5 Cycles
	INX				; 2 Cycles
	BNE	scroll_l_2_loop		; 3 Cycles
					; = 10455 Cycles

;;------------------------------------------------------------------------------
;; Verschiebt auch die Sprite-Pointer (die sind naemlich in der Schleife da oben
;; durcheinandergeraten).
;;------------------------------------------------------------------------------
	LDX	#$08
sprite_l_dings_loop:
	LDA	$07E8, X
	STA	$0BE8, X
	DEX
	BNE	sprite_l_dings_loop

;;------------------------------------------------------------------------------
;; Modifiziert den Code dieser Subroutine dahingehend, dass beim naechsten Auf-
;; ruf vom derzeit aktiven in den inaktiven Buffer geschrieben wird.
;;------------------------------------------------------------------------------
modify_yourself:
	LDA	scroll_l_2_loop + 02
	CMP	#$04
	BNE	mod_to_page_1
	LDA	#$08
	STA	scroll_l_2_loop + 02
	LDA	#$04
	STA	scroll_l_2_loop + 05
	LDA	#$09
	STA	scroll_l_2_loop + 08
	LDA	#$05
	STA	scroll_l_2_loop + 11
	LDA	#$0A
	STA	scroll_l_2_loop + 14
	LDA	#$06
	STA	scroll_l_2_loop + 17
	LDA	#$0B
	STA	scroll_l_2_loop + 20
	STA	sprite_l_dings_loop + 2
	LDA	#$07
	STA	scroll_l_2_loop + 23
	STA	sprite_l_dings_loop + 5
	JMP	scroll_l_2_end

mod_to_page_1:
	LDA	#$04
	STA	scroll_l_2_loop + 02
	LDA	#$08
	STA	scroll_l_2_loop + 05
	LDA	#$05
	STA	scroll_l_2_loop + 08
	LDA	#$09
	STA	scroll_l_2_loop + 11
	LDA	#$06
	STA	scroll_l_2_loop + 14
	LDA	#$0A
	STA	scroll_l_2_loop + 17
	LDA	#$07
	STA	scroll_l_2_loop + 20
	STA	sprite_l_dings_loop + 2
	LDA	#$0B
	STA	scroll_l_2_loop + 23
	STA	sprite_l_dings_loop + 5

scroll_l_2_end:
	PLA
	TAX
	PLA
	PLP
	RTS



;;==============================================================================
;; Sichert Spalte 0 des Bildschirmspeichers und modifiziert den eigenen Code
;; anschliessen so, dass beim naechsten Aufruf aus dem anderen Bildschirmpuffer
;; gelesen wird.
;;==============================================================================
save_left:
	PHP
	PHA
	TXA
	PHA

;;------------------------------------------------------------------------------
;; Sichert zu erst das erste Zeichen der _letzten_ Zeile (ich zaehle
;; rueckwaerts).
;; Modifiziert dann die Adresse, von der die Zeichen geladen werden, indem von
;; ihr $28/40 (= eine Zeile) subtrahiert wird.
;;------------------------------------------------------------------------------
	LDX	#$18		; 25 (-1) Zeilen zu sichern
save_left_loop:
	LDA	$07C0		; Erstes Zeichen der letzten Zeile.
	STA	column, X
	;; Die Adresse, von der ein Byte geladen werden soll um $28/40 ( = eine
	;; Zeile) erniedrigen. 16-Bit-Subtraktion.
	SEC
	LDA	save_left_loop + 1
	SBC	#$28
	STA	save_left_loop + 1
	LDA	save_left_loop + 2
	SBC	#$00
	STA	save_left_loop + 2

	DEX
	BPL	save_left_loop:

;;------------------------------------------------------------------------------
;; Modifiziert den Code folgendermassen:
;; Das LSB wird auf jeden Fall auf $C0 gesetzt (egal ob Puffer 1 oder 2).
;; War ist das MSB jetzt $03 (entspricht Puffer 1 bei $0400) setze es auf $0B
;; (letzte Zeile in Puffer 2).
;; Ansonsten setze es auf $07 (letzte Zeile in Puffer 1).
;;------------------------------------------------------------------------------
mod_load_adr:
	LDA	#$C0
	STA	save_left_loop + 1
	LDA	save_left_loop + 2
	;; $03 anstatt $04, da im loop oben von $0400 nochmal $28 subtrahiert
	;; werden.
	CMP	#$03
	BNE	set_save_to_page2
	LDA	#$0B
	STA	save_left_loop + 2
	JMP	save_left_end
set_save_to_page2:
	LDA	#$07
	STA	save_left_loop + 2

save_left_end:
	PLA
	TAX
	PLA
	PLP
	RTS



;;==============================================================================
;; Schreibt die gesicherte Spalte 0 in Spalte 39.
;;==============================================================================
restore_right:
	PHP
	PHA
	TXA
	PHA

;;------------------------------------------------------------------------------
;; Stellt zuerst das letzte Zeichen der _letzten_ Zeile (ich zaehle
;; rueckwaerts) wieder her.
;; Modifiziert dann die Adresse, von der die Zeichen geladen werden,
;; folgendermassen:
;; Ziehe zuerst vom LSB der Adresse 40 (eine Zeile) ab. Ist das Ergebnis
;; negativ, dann dekrementiere das MSB um eins.
;;------------------------------------------------------------------------------
	LDX	#$18
restore_right_loop:
	;; Stellt ein Zeichen wieder her
	LDA	column, X
	STA	$0BE7

	;; Die Adresse, an die ein Byte geschrieben werden soll um $28/40
	;; ( = eine Zeile) erniedrigen.
	SEC
	LDA	restore_right_loop + 4
	SBC	#$28
	STA	restore_right_loop + 4
	LDA	restore_right_loop + 5
	SBC	#$00
	STA	restore_right_loop + 5
	DEX
	BPL	restore_right_loop:

;;------------------------------------------------------------------------------
;; Modifiziert den Code folgendermassen:
;; Das LSB wird auf jeden Fall auf $E7 gesetzt (egal ob Puffer 1 oder 2).
;; Ist das MSB jetzt $03 (entspricht Puffer 1 bei $0400) setze es auf $0B
;; (letzte Zeile in Puffer 2).
;; Ansonsten setze es auf $07 (letzte Zeile in Puffer 1).
;;------------------------------------------------------------------------------
mod_save_adr:
	LDA	#$E7
	STA	restore_right_loop + 4
	;; $03 anstatt $04, da im loop oben von $0427 nochmal $28 subtrahiert
	;; werden.
	LDA	#$03
	CMP	restore_right_loop + 5
	BNE	set_restore_to_page2
	LDA	#$0B
	STA	restore_right_loop + 5
	JMP	restore_right_end
set_restore_to_page2:
	LDA	#$07
	STA	restore_right_loop + 5

restore_right_end:
	PLA
	TAX
	PLA
	PLP
	RTS



;;==============================================================================
;; Gibt einen Testtext am Anfang von Zeile 12 ($05E0) aus.
;;==============================================================================
write_testline:
	PHP
	PHA
	TXA
	PHA

	LDX	#$00
write_loop:
	LDA	text, X
	BEQ	end_write
	STA	$05E0, X
	INX
	JMP	write_loop

end_write:
	PLA
	TAX
	PLA
	PLP
	RTS



;;==============================================================================
;; Ueberprueft den Joystick in Port 1.
;; Die Bits sind bei Aktivitaet auf 0 gesetzt.
;; Bit 0: oben
;; Bit 1: unten
;; Bit 2: links
;; Bit 3: rechts
;; Bit 4: Feuer
;; Vorgehensweise: Den Akkumulator rechtsrum durch das Carry-Flag shiften und
;; dann jeweils verzweigen.
;;==============================================================================
check_stick1:
	PHP
	PHA

	LDA	$DC00			; Port 1 einlesen

	LSR				; oben
	BCC	oben			; Wenn 0 dann ja
	LSR				; unten
	BCC	unten
	LSR				; links
	BCC	links
	LSR				; rechts
	BCC	rechts
	LSR				; Feuerknopf
	BCC	stick1_fired
	JMP	check_stick_ende

;;------------------------------------------------------------------------------
;; Ueberprueft auch oben/links und oben/rechts und Feuer.
;;------------------------------------------------------------------------------
oben:
	LSR
	LSR
	BCC	oben_links
	LSR
	BCC	oben_rechts

	JSR	move_up
	LSR
	BCC	stick1_fired
	JMP	check_stick_ende

oben_links:
	JSR	move_up_left
	LSR
	LSR
	BCC	stick1_fired
	JMP	check_stick_ende

oben_rechts:
	JSR	move_up_right
	LSR
	BCC	stick1_fired
	JMP	check_stick_ende

;;------------------------------------------------------------------------------
;; Ueberprueft auch unten/links und unten/rechts und Feuer.
;;------------------------------------------------------------------------------
unten:
	LSR
	BCC	unten_links
	LSR
	BCC	unten_rechts

	JSR	move_down
	LSR
	BCC	stick1_fired
	JMP	check_stick_ende

unten_links:
	JSR	move_down_left
	LSR
	LSR
	BCC	stick1_fired
	JMP	check_stick_ende

unten_rechts:
	JSR	move_down_right
	LSR
	BCC	stick1_fired
	JMP	check_stick_ende

links:
	JSR	move_left
	LSR
	LSR
	BCC	stick1_fired
	JMP	check_stick_ende

rechts:
	JSR	move_right
	LSR
	BCC	stick1_fired
	JMP	check_stick_ende

stick1_fired:
	JSR	fire

check_stick_ende:
	PLA
	PLP
	RTS




;;==============================================================================
;; Bewegung nach oben
;;==============================================================================
move_up:
	PHP
	PHA

	DEC	$D001

	PLA
	PLP
	RTS



;;==============================================================================
;; Bewegung nach oben/links
;;==============================================================================
move_up_left:
	PHP
	PHA

	DEC	$D000
	DEC	$D001

	PLA
	PLP
	RTS



;;==============================================================================
;; Bewegung nach oben/rechts
;;==============================================================================
move_up_right:
	PHP
	PHA

	INC	$D000
	DEC	$D001

	PLA
	PLP
	RTS



;;==============================================================================
;; Bewegung nach unten
;;==============================================================================
move_down:
	PHP
	PHA

	INC	$D001

	PLA
	PLP
	RTS



;;==============================================================================
;; Bewegung nach unten/links
;;==============================================================================
move_down_left:
	PHP
	PHA

	DEC	$D000
	INC	$D001

	PLA
	PLP
	RTS



;;==============================================================================
;; Bewegung nach unten/rechts
;;==============================================================================
move_down_right:
	PHP
	PHA

	INC	$D000
	INC	$D001

	PLA
	PLP
	RTS



;;==============================================================================
;; Bewegung nach links
;;==============================================================================
move_left:
	PHP
	PHA

	DEC	$D000

	PLA
	PLP
	RTS



;;==============================================================================
;; Bewegung nach rechts
;;==============================================================================
move_right:
	PHP
	PHA

	INC	$D000

	PLA
	PLP
	RTS



;;==============================================================================
;; Feuern
;;==============================================================================
fire:
	PHA

	INC	$D027

	PLA
	RTS



;;==============================================================================
;; Setzt den Zeiger fuer Sprite 0 auf $2000
;; Setzt X- und Y-Koordinate auf jeweils 0.
;;==============================================================================
init_sprite0:
	PHA

	LDA     #$80
	STA	$07F8

	LDA	#$80
	STA	$D000
	STA	$D001

	LDA	#01
	STA	$D027

	LDA	$D015
	ORA	#$01
	STA	$D015

	PLA
	RTS


;;------------------------------------------------------------------------------
;; 0-terminierter String.
;;------------------------------------------------------------------------------
text	!byte $13, $0F, $06, $14, $13, $03, $12, $0F, $0C, $0C, $09, $0E, $07, $00

;;------------------------------------------------------------------------------
;; Reserviert platz fuer eine Spalte des Bildschirmspeichers (25 Bytes)
;;------------------------------------------------------------------------------
column	!fill	$19

;;------------------------------------------------------------------------------
;; Sprite Daten fuer Sprite 0.
;; Zu laden ins RAM an Position $2000 (8192).
;;------------------------------------------------------------------------------

*=$2000
spr_0	!byte	$00, $FF, $00, $00, $FF, $00, $00, $FF
spr_1	!byte	$00, $00, $FF, $00, $00, $FF, $00, $00
spr_2	!byte	$FF, $00, $00, $FF, $00, $FF, $FF, $FF
spr_3	!byte	$FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF
spr_4	!byte	$FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF
spr_5	!byte	$FF, $FF, $00, $FF, $00, $00, $FF, $00
spr_6	!byte	$00, $FF, $00, $00, $FF, $00, $00, $FF
spr_7	!byte	$00, $00, $FF, $00, $00, $FF, $00
